@@ -6,7 +6,7 @@ from textwrap import dedent
 import unittest
 from unittest.mock import MagicMock
 
-from green.config import default_args
+from green.config import get_default_args
 from green.loader import GreenTestLoader
 from green.runner import run
 from green.suite import GreenTestSuite
@@ -23,6 +23,7 @@ class TestGreenTestSuite(unittest.TestCase):
         """
         Passing in default arguments causes attributes to be set.
         """
+        default_args = get_default_args()
         gts = GreenTestSuite(args=default_args)
         self.assertEqual(gts.allow_stdout, default_args.allow_stdout)
 
@@ -31,7 +32,7 @@ class TestGreenTestSuite(unittest.TestCase):
         When result.shouldStop == True, the suite should exit early.
         """
         mock_test = MagicMock()
-        gts = GreenTestSuite(args=default_args)
+        gts = GreenTestSuite(args=get_default_args())
         gts._tests = (mock_test,)
         mock_result = MagicMock()
         mock_result.shouldStop = True
@@ -43,7 +44,7 @@ class TestGreenTestSuite(unittest.TestCase):
         """
         mock_test = MagicMock()
         mock_test.__iter__.side_effect = TypeError
-        gts = GreenTestSuite(args=default_args)
+        gts = GreenTestSuite(args=get_default_args())
         gts._tests = (mock_test,)
         mock_result = MagicMock()
         mock_result._moduleSetUpFailed = True
@@ -57,7 +58,7 @@ class TestGreenTestSuite(unittest.TestCase):
         mock_module = MagicMock()
         mock_test = MagicMock()
         mock_err = MagicMock()
-        args = copy.deepcopy(default_args)
+        args = copy.deepcopy(get_default_args())
         gts = GreenTestSuite(args=args)
         gts._get_previous_module = mock_module
         mock_result = MagicMock()
@@ -75,7 +76,7 @@ class TestGreenTestSuite(unittest.TestCase):
         mock_test._testMethodName = "test_hello"
         mock_test2 = MagicMock()
         mock_test2._testMethodName = "test_goodbye"
-        args = copy.deepcopy(default_args)
+        args = copy.deepcopy(get_default_args())
         args.test_pattern = "_good*"
         gts = GreenTestSuite(args=args)
         gts.addTest(mock_test)
@@ -100,7 +101,7 @@ class TestGreenTestSuite(unittest.TestCase):
         """
         If SkipTest is raised in setUpClass, then the test gets skipped
         """
-        gts = GreenTestSuite(args=default_args)
+        gts = GreenTestSuite(args=get_default_args())
         mock_test = MagicMock()
         mock_result = MagicMock()
         mock_class = MagicMock()
@@ -129,7 +130,7 @@ class TestFunctional(unittest.TestCase):
         cls.startdir = None
 
     def setUp(self):
-        self.args = copy.deepcopy(default_args)
+        self.args = copy.deepcopy(get_default_args())
         self.stream = StringIO()
         self.tmpdir = tempfile.mkdtemp()
         self.loader = GreenTestLoader()
@@ -181,7 +182,7 @@ class TestModuleTeardown(unittest.TestCase):
         cls.startdir = None
 
     def setUp(self):
-        self.args = copy.deepcopy(default_args)
+        self.args = copy.deepcopy(get_default_args())
         self.stream = StringIO()
         self.tmpdir = tempfile.mkdtemp()
         self.loader = GreenTestLoader()
